@@ -2,8 +2,6 @@ from __future__ import division, absolute_import, unicode_literals
 
 from PyQt4 import QtCore, QtGui
 
-from cola.compat import ustr
-
 have_pygments = True
 try:
     from pygments.styles import get_style_by_name
@@ -61,7 +59,7 @@ def highlight_document(edit, filename):
         token_formats[token] = format
         return format
 
-    text = ustr(doc.toPlainText())
+    text = doc.toPlainText()
 
     block_count = 0
     block = doc.firstBlock()
@@ -85,7 +83,7 @@ def highlight_document(edit, filename):
             if block_len == 0:
                 block.layout().setAdditionalFormats(block_formats)
                 doc.markContentsDirty(block.position(), block.length())
-                block = block.next()
+                block = block.next()  # pylint: disable=next-method-called
                 block_pos = 0
                 block_len = block.length()
                 block_formats = []
