@@ -10,7 +10,6 @@ from cola import gitcmds
 from cola import icons
 from cola import qtutils
 from cola import utils
-from cola.compat import ustr
 from cola.i18n import N_
 from cola.interaction import Interaction
 from cola.models import main
@@ -274,7 +273,7 @@ class RemoteActionDialog(standard.Dialog):
         if item:
             self.remotes.setItemSelected(item, True)
             self.remotes.setCurrentItem(item)
-            self.set_remote_name(ustr(item.text()))
+            self.set_remote_name(item.text())
             return True
         else:
             return False
@@ -353,9 +352,9 @@ class RemoteActionDialog(standard.Dialog):
 
     def common_args(self):
         """Returns git arguments common to fetch/push/pulll"""
-        remote_name = ustr(self.remote_name.text())
-        local_branch = ustr(self.local_branch.text())
-        remote_branch = ustr(self.remote_branch.text())
+        remote_name = self.remote_name.text()
+        local_branch = self.local_branch.text()
+        remote_branch = self.remote_branch.text()
 
         ffwd_only = self.ffwd_only_checkbox.isChecked()
         rebase = self.rebase_checkbox.isChecked()
@@ -389,7 +388,7 @@ class RemoteActionDialog(standard.Dialog):
         else: # if action == PULL:
             model_action = self.model.pull
 
-        remote_name = ustr(self.remote_name.text())
+        remote_name = self.remote_name.text()
         if not remote_name:
             errmsg = N_('No repository selected.')
             Interaction.log(errmsg)
@@ -399,8 +398,8 @@ class RemoteActionDialog(standard.Dialog):
                                                        self.model.remotes)
 
         # Check if we're about to create a new branch and warn.
-        remote_branch = ustr(self.remote_branch.text())
-        local_branch = ustr(self.local_branch.text())
+        remote_branch = self.remote_branch.text()
+        local_branch = self.local_branch.text()
 
         if action == PUSH and not remote_branch:
             branch = local_branch
